/**
 * 
 */
package com.up.zpb.util;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author mingguangz yyyy-MM-dd HH:mm:ss
 */
public final class DateUtils {
	private static final Logger logger = LoggerFactory
			.getLogger(DateUtils.class);
	private static final TimeZone defaultTimeZone = TimeZone.getDefault();

	private DateUtils() {
	}

	/**
	 * 
	 * @return The current date time
	 */
	public static Date now() {
		return new Date();
	}

	/**
	 * 
	 * @param pattern
	 * @return The current date time
	 */
	public static String now(String pattern) {
		return format(now(), pattern);
	}

	/**
	 * 
	 * @param pattern
	 * @param timeZone
	 * @return The current date time
	 */
	public static String now(String pattern, TimeZone timeZone) {
		return format(now(), pattern, timeZone);
	}

	/**
	 * Adds a number of years to a date returning a new object. The original
	 * date object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addYears(Date date, int amount) {
		return add(date, Calendar.YEAR, amount);
	}

	/**
	 * Adds a number of months to a date returning a new object. The original
	 * date object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addMonths(Date date, int amount) {
		return add(date, Calendar.MONTH, amount);
	}

	/**
	 * Adds a number of weeks to a date returning a new object. The original
	 * date object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addWeeks(Date date, int amount) {
		return add(date, Calendar.WEEK_OF_YEAR, amount);
	}

	/**
	 * Adds a number of days to a date returning a new object. The original date
	 * object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addDays(Date date, int amount) {
		return add(date, Calendar.DAY_OF_MONTH, amount);
	}

	/**
	 * Adds a number of hours to a date returning a new object. The original
	 * date object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addHours(Date date, int amount) {
		return add(date, Calendar.HOUR_OF_DAY, amount);
	}

	/**
	 * Adds a number of minutes to a date returning a new object. The original
	 * date object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addMinutes(Date date, int amount) {
		return add(date, Calendar.MINUTE, amount);
	}

	/**
	 * Adds a number of seconds to a date returning a new object. The original
	 * date object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addSeconds(Date date, int amount) {
		return add(date, Calendar.SECOND, amount);
	}

	/**
	 * Adds a number of milliseconds to a date returning a new object. The
	 * original date object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addMilliseconds(Date date, int amount) {
		return add(date, Calendar.MILLISECOND, amount);
	}

	/**
	 * Adds to a date returning a new object. The original date object is
	 * unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param calendarField
	 *            the calendar field to add to
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	private static Date add(Date date, int calendarField, int amount) {
		if (date == null) {
			throw new IllegalArgumentException("The date must not be null");
		}
		Calendar c = Calendar.getInstance();
		c.setTime(date);
		c.add(calendarField, amount);
		return c.getTime();
	}

	/**
	 * <p>
	 * Formats a date/time into a specific pattern.
	 * </p>
	 * 
	 * @param date
	 *            the date to format
	 * @param pattern
	 *            the pattern to use to format the date
	 *            如:"yyyy/MM/dd","yyyy-MM-dd"
	 *            ,"yyyyMMdd","yyyy年MM月dd日 HH时mm分ss秒", "yyyy-MM-dd HH:mm:ss"等等
	 * @return the formatted date
	 */
	public static String format(Date date, String pattern) {
		return format(date, pattern, defaultTimeZone);
	}

	/**
	 * 
	 * @param date
	 * @param pattern
	 * @return A Date parsed from the string.
	 */
	public static Date format(String date, String pattern) {
		return format(date, pattern, defaultTimeZone);
	}

	/**
	 * 
	 * @param date
	 * @param currentPattern
	 * @param targetPattern
	 * @return the formatted date
	 */
	public static String format(String date, String currentPattern,
			String targetPattern) {
		return format(date, currentPattern, targetPattern, defaultTimeZone);
	}

	/**
	 * <p>
	 * Formats a date/time into a specific pattern in a time zone.
	 * </p>
	 * 
	 * @param date
	 *            the date to format
	 * @param pattern
	 *            the pattern to use to format the date
	 * @param timeZone
	 *            the time zone to use, may be <code>null</code>
	 * @return the formatted date
	 */
	public static String format(Date date, String pattern, TimeZone timeZone) {
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat(pattern);
		if (timeZone != null)
			simpleDateFormat.setTimeZone(timeZone);
		return simpleDateFormat.format(date);
	}

	/**
	 * 
	 * @param date
	 * @param pattern
	 * @param timeZone
	 * @return A Date parsed from the string.
	 */
	public static Date format(String date, String pattern, TimeZone timeZone) {
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat(pattern);
		if (timeZone != null)
			simpleDateFormat.setTimeZone(timeZone);
		try {
			return simpleDateFormat.parse(date);
		} catch (ParseException e) {
			// e.printStackTrace();
			logger.error(e.getMessage(), e);
		}
		return null;
	}

	/**
	 * 
	 * @param date
	 * @param currentPattern
	 * @param targetPattern
	 * @param timeZone
	 * @return
	 */
	public static String format(String date, String currentPattern,
			String targetPattern, TimeZone timeZone) {
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat(currentPattern);
		if (timeZone != null)
			simpleDateFormat.setTimeZone(timeZone);
		try {
			return format(simpleDateFormat.parse(date), targetPattern, timeZone);
		} catch (ParseException e) {
			logger.error(e.getMessage(), e);
		}
		return null;
	}

	/**
	 * 取得给定日期在当前年中的第几个星期
	 * 
	 * @param date
	 * @return
	 */
	public static int getWeekOfYear(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return cal.get(Calendar.WEEK_OF_YEAR);
	}

	/**
	 * 取得给定日期是当前月中的第几个星期
	 * 
	 * @param date
	 * @return
	 */
	public static int getWeekOfMonth(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return cal.get(Calendar.WEEK_OF_MONTH);
	}

	/**
	 * 取得给定日期是星期几
	 * 
	 * @param date
	 * @return 1~7
	 */
	public static String getWeek(Date date) {
		String[] weeks = { "星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六" };
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		int week_index = cal.get(Calendar.DAY_OF_WEEK);
		return weeks[week_index - 1];
	}

	/**
	 * 取得一年内有多少个星期
	 * 
	 * @param year
	 * @return
	 */
	public static int getWeeks(int year) throws ParseException {
		Calendar cal = Calendar.getInstance();
		cal.setTime(format(year + "-12-31", "yyyy-MM-dd"));
		return cal.get(Calendar.WEEK_OF_YEAR);
	}

	/**
	 * 计算出给定两个日期的时间间隔
	 * 
	 * @param startdate
	 * @param enddate
	 * @param pattern
	 *            时间间隔的单位("d"天 ，"h"小时，"m"分，"s"秒，"ms"豪秒)
	 * @return
	 */
	public static long dateDifference(Date startdate, Date enddate,
			String pattern) {
		long diffMillis = enddate.getTime() - startdate.getTime();
		if ("ms".equals(pattern))
			return diffMillis;
		else if ("s".equals(pattern))
			return diffMillis / 1000;
		else if ("m".equals(pattern))
			return diffMillis / (1000 * 60);
		else if ("h".equals(pattern))
			return diffMillis / 0x36ee80;
		else if ("d".equals(pattern))
			return diffMillis / 0x5265c00;
		else
			return 0;
	}

	/**
	 * 得到某个日期当月的天数
	 * 
	 * @param year
	 * @param month
	 * @return
	 */
	public static int getDaysOfMonth(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(getLastDayOfMonth(date));
		return cal.get(Calendar.DATE);
	}

	/**
	 * 得到某个日期的当月最后一天
	 * 
	 * @param date
	 * @return
	 */
	public static Date getLastDayOfMonth(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return addDays(addMonths(addDays(date, 1 - cal.get(Calendar.DATE)), 1),
				-1);
	}

	public static int getAgeByBirthday(Date birthday) {
		return getAgeByBirthday(birthday, new Date());
	}

	public static int getAgeByBirthday(Date birthday, Date endDate) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(endDate);
		if (cal.before(birthday)) {
			throw new IllegalArgumentException(
					"The birthDay is before Now.It's unbelievable!");
		}
		int yearNow = cal.get(Calendar.YEAR);
		int monthNow = cal.get(Calendar.MONTH) + 1;
		int dayOfMonthNow = cal.get(Calendar.DAY_OF_MONTH);

		cal.setTime(birthday);
		int yearBirth = cal.get(Calendar.YEAR);
		int monthBirth = cal.get(Calendar.MONTH) + 1;
		int dayOfMonthBirth = cal.get(Calendar.DAY_OF_MONTH);

		int age = yearNow - yearBirth;

		if (monthNow <= monthBirth) {
			if (monthNow == monthBirth) {
				if (dayOfMonthNow < dayOfMonthBirth) {
					age--;
				}
			} else {
				age--;
			}
		}
		return age;
	}
	
	public static Date beginTimeHelper(Date date) {
		try {
			if (null != date) {
				SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd hh:mm:ss");
				SimpleDateFormat sdfShort = new SimpleDateFormat("yyyy/MM/dd");
				return sdf.parse(sdfShort.format(date) + " 00:00:00");
			}
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return date;
	}
	
	public static Date endTimeHelper(Date date) {
		try {
			if (null != date) {
				SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd hh:mm:ss");
				SimpleDateFormat sdfShort = new SimpleDateFormat("yyyy/MM/dd");
				return sdf.parse(sdfShort.format(date) + " 23:59:59");
			}
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return date;
	}
	
	
	public static int getDatePoor(Date endDate, Date nowDate) {
		 
	    long nd = 1000 * 24 * 60 * 60;
//	    long nh = 1000 * 60 * 60;
//	    long nm = 1000 * 60;
	    // long ns = 1000;
	    // 获得两个时间的毫秒时间差异
	    long diff = endDate.getTime() - nowDate.getTime();
//	    // 计算差多少天
	    long day = diff / nd;
	    // 计算差多少小时
//	    long hour = diff % nd / nh;
	    // 计算差多少分钟
//	    long min = diff % nd % nh / nm;
	    // 计算差多少秒//输出结果
	    // long sec = diff % nd % nh % nm / ns;
	    return (int) day;
	}

	/**
	 * 时间转换 xxx时间之内
	 * 
	 * @param date
	 * @return
	 * 
	 */
	public static String fromToday(Date date) {
		Long timestamp = (System.currentTimeMillis() - date.getTime()) / 60000;
		if (timestamp == 0) {
			return "1分钟之内";
		} else if (timestamp < 3) {
			return "3分钟之内";
		} else if (timestamp < 5) {
			return "5分钟之内";
		} else if (timestamp < 10) {
			return "10分钟之内";
		} else if (timestamp < 30) {
			return "30分钟之内";
		} else if (timestamp < 60) {
			return "1小时内";
		} else if (timestamp < 120) {
			return "2小时内";
		}
		SimpleDateFormat times = new SimpleDateFormat("yyyy-MM-dd");
		String t = times.format(System.currentTimeMillis());
		return String.valueOf(t);
	}

	/**
	 * 根据两个日期的相差天数
	 * 
	 * @param d1
	 * @param d2
	 * @return
	 */
	public static int getElapsedMonths(Date d1, Date d2) {
		return elapsed(d1, d2, Calendar.DAY_OF_MONTH);
	}

	/**
	 * All elaspsed types
	 * 
	 * @param d1   Date
	 * @param d2   Date
	 * @param type int (Calendar.FIELD_NAME)
	 * 
	 * @return int number of elapsed "type"
	 */
	private static int elapsed(Date d1, Date d2, int type) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(d1);
		GregorianCalendar g1 = new GregorianCalendar(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH),
				cal.get(Calendar.DATE));
		cal.setTime(d2);
		GregorianCalendar g2 = new GregorianCalendar(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH),
				cal.get(Calendar.DATE));
		return elapsed(g1, g2, type);
	}

	/**
	 * All elaspsed types
	 * 
	 * @param g1   GregorianCalendar
	 * @param g2   GregorianCalendar
	 * @param type int (Calendar.FIELD_NAME)
	 * 
	 * @return int number of elapsed "type"
	 */
	private static int elapsed(GregorianCalendar g1, GregorianCalendar g2, int type) {
		GregorianCalendar gc1, gc2;
		int elapsed = 0;
		// Create copies since we will be clearing/adding
		if (g2.after(g1)) {
			gc2 = (GregorianCalendar) g2.clone();
			gc1 = (GregorianCalendar) g1.clone();
		} else {
			gc2 = (GregorianCalendar) g1.clone();
			gc1 = (GregorianCalendar) g2.clone();
		}
		if (type == Calendar.MONTH || type == Calendar.YEAR) {
			gc1.clear(Calendar.DATE);
			gc2.clear(Calendar.DATE);
		}
		if (type == Calendar.YEAR) {
			gc1.clear(Calendar.MONTH);
			gc2.clear(Calendar.MONTH);
		}
		while (gc1.before(gc2)) {
			gc1.add(type, 1);
			elapsed++;
		}
		return elapsed;
	}
	
	// 获得某天最小时间 2019-08-27 00:00:00
	public static Date getStartOfDay(Date date) {
		LocalDateTime localDateTime = LocalDateTime.ofInstant(Instant.ofEpochMilli(date.getTime()), ZoneId.systemDefault());
		LocalDateTime startOfDay = localDateTime.with(LocalTime.MIN);
		return Date.from(startOfDay.atZone(ZoneId.systemDefault()).toInstant());
	}
	
	 // 获得某天最大时间 2019-08-27 23:59:59
    public static Date getEndOfDay(Date date) {
        LocalDateTime localDateTime = LocalDateTime.ofInstant(Instant.ofEpochMilli(date.getTime()), ZoneId.systemDefault());;
        LocalDateTime endOfDay = localDateTime.with(LocalTime.MAX);
        return Date.from(endOfDay.atZone(ZoneId.systemDefault()).toInstant());
    }
    
    
    
    
    public static void main(String[] args) {
		/*String content="刚刚有经纪⼈在您发布的【${appTitle}】房/客源上传了到访信息，快去查看吧。如果信息有误，请拨打优铺客服电话4008988808解决。";
		if(content.contains("${appTitle}")){
			content=content.replace("${appTitle}", "wan大广场地地道道地地道道");
		}
		System.out.println("content:"+content);*/
    	System.out.println();
	}
	
	
}
